import { PassengerOrientedCarpoolPathCompleter } from '@modules/ad/core/application/queries/match/completer/passenger-oriented-carpool-path.completer';
import { MatchQuery } from '@modules/ad/core/application/queries/match/match.query';
import { AlgorithmType } from '@modules/ad/core/application/types/algorithm.types';
import { Waypoint } from '@modules/ad/core/application/types/waypoint.type';
import { Frequency, Role } from '@modules/ad/core/domain/ad.types';
import { CandidateEntity } from '@modules/ad/core/domain/candidate.entity';

const originWaypoint: Waypoint = {
  position: 0,
  lat: 48.689445,
  lon: 6.17651,
  houseNumber: '5',
  street: 'Avenue Foch',
  locality: 'Nancy',
  postalCode: '54000',
  country: 'France',
};
const destinationWaypoint: Waypoint = {
  position: 1,
  lat: 48.8566,
  lon: 2.3522,
  locality: 'Paris',
  postalCode: '75000',
  country: 'France',
};

const matchQuery = new MatchQuery(
  {
    algorithmType: AlgorithmType.PASSENGER_ORIENTED,
    driver: true,
    passenger: true,
    frequency: Frequency.PUNCTUAL,
    fromDate: '2023-08-28',
    toDate: '2023-08-28',
    schedule: [
      {
        time: '07:05',
      },
    ],
    strict: false,
    waypoints: [originWaypoint, destinationWaypoint],
  },
  {
    getBasic: jest.fn(),
  },
);

const candidates: CandidateEntity[] = [
  CandidateEntity.create({
    id: 'cc260669-1c6d-441f-80a5-19cd59afb777',
    role: Role.DRIVER,
    driverWaypoints: [
      {
        lat: 48.678454,
        lon: 6.189745,
      },
      {
        lat: 48.84877,
        lon: 2.398457,
      },
    ],
    passengerWaypoints: [
      {
        lat: 48.689445,
        lon: 6.17651,
      },
      {
        lat: 48.8566,
        lon: 2.3522,
      },
    ],
    driverDistance: 350145,
    driverDuration: 13548,
    spacetimeDetourRatio: {
      maxDistanceDetourRatio: 0.3,
      maxDurationDetourRatio: 0.3,
    },
  }),
  CandidateEntity.create({
    id: '5600ccfb-ab69-4d03-aa30-0fbe84fcedc0',
    role: Role.PASSENGER,
    driverWaypoints: [
      {
        lat: 48.689445,
        lon: 6.17651,
      },
      {
        lat: 48.8566,
        lon: 2.3522,
      },
    ],
    passengerWaypoints: [
      {
        lat: 48.689445,
        lon: 6.17651,
      },
      {
        lat: 48.8566,
        lon: 2.3522,
      },
    ],
    driverDistance: 350145,
    driverDuration: 13548,
    spacetimeDetourRatio: {
      maxDistanceDetourRatio: 0.3,
      maxDurationDetourRatio: 0.3,
    },
  }),
];

describe('Passenger oriented carpool path completer', () => {
  it('should complete candidates', async () => {
    const passengerOrientedCarpoolPathCompleter: PassengerOrientedCarpoolPathCompleter =
      new PassengerOrientedCarpoolPathCompleter(matchQuery);
    const completedCandidates: CandidateEntity[] =
      await passengerOrientedCarpoolPathCompleter.complete(candidates);
    expect(completedCandidates.length).toBe(2);
  });
});
