import { GeorouterPort } from '../application/ports/georouter.port';
import { GeorouterSettings } from '../application/types/georouter-settings.type';
import { PointProps } from './value-objects/point.value-object';

// All properties that a Route has
export interface RouteProps {
  distance: number;
  duration: number;
  fwdAzimuth: number;
  backAzimuth: number;
  distanceAzimuth: number;
  points: PointProps[];
}

// Properties that are needed for a Route creation
export interface CreateRouteProps {
  waypoints: PointProps[];
  georouter: GeorouterPort;
  georouterSettings: GeorouterSettings;
}

// Types used outside the domain
export type Route = {
  distance: number;
  duration: number;
  fwdAzimuth: number;
  backAzimuth: number;
  distanceAzimuth: number;
  points: Point[];
  steps?: Step[];
};

export type Point = {
  lon: number;
  lat: number;
};

export type Step = Point & {
  duration: number;
  distance?: number;
};
